// ignore: implementation_imports, import_of_legacy_library_into_null_safe
import 'package:build/src/builder/build_step.dart';
import 'package:analyzer/dart/element/element.dart' as ele;

// ignore: import_of_legacy_library_into_null_safe
import 'package:source_gen/source_gen.dart';

// ignore: import_of_legacy_library_into_null_safe
import 'package:build/build.dart';
import 'dart:io';
import 'project.dart';
import 'tpl/tpl.dart' as tpl;

class ProjectGenerator extends GeneratorForAnnotation<GProject> {
  final separator = Platform.pathSeparator;
  final libs = [
    LibInfo(name: 'flutter_cupertino_localizations', version: '1.0.1'),
    LibInfo(name: 'redux', version: '5.0.0'),
    LibInfo(name: 'synchronized', version: '3.0.0'),
    LibInfo(name: 'shared_preferences', version: '2.0.3'),
    LibInfo(name: 'path_provider', version: '2.0.1'),
    LibInfo(name: 'dio', version: '4.0.0-beta3'),
  ];

  String readPubspec(String path) {
    var sb = StringBuffer();
    bool isDev = false;
    try {
      List<String> lines = File(path).readAsLinesSync();
      lines.forEach((line) {
        if (line.startsWith('dependencies:')) {
          isDev = false;
        }
        if (line.startsWith('dev_dependencies:')) {
          isDev = true;
        }
        if (!isDev && line.startsWith('  flutter:')) {
          libs.forEach((lib) {
            sb.writeln('''  ${lib.name}: ^${lib.version}''');
          });
        }
        sb.writeln(line);
      });
      return sb.toString();
    } catch (e) {
      stdout.writeln('文件读取失败! $e');
    }
    return sb.toString();
  }

  @override
  generateForAnnotatedElement(
      ele.Element element, ConstantReader annotation, BuildStep buildStep) {
    var init = false;
    var enable = false;
    try {
      enable = annotation.read('enable').boolValue;
    } catch (e) {}
    if (!enable) return null;
    try {
      init = annotation.read('init').boolValue;
    } catch (e) {}
    if (!init) {
      return null;
    }
    var package = buildStep.inputId.package;
    //初始化第三方库. build.sh
    File('${Directory.current.path}${separator}pubspec.yaml').writeAsStringSync(
        readPubspec('${Directory.current.path}${separator}pubspec.yaml'));
    File('${Directory.current.path}${separator}build.sh')
      ..createSync()
      ..writeAsStringSync(tpl.Tpl.buildSh(package: package));
    //删除main.dart, test
    File('${Directory.current.path}${separator}lib${separator}main.dart')
        .deleteSync();
    File('${Directory.current.path}${separator}test')
        .deleteSync(recursive: true);
    tpl.Tpl.buildCode.forEach((info) {
      var filePath =
          '${Directory.current.path}${separator}lib$separator${info.filePath}';
      File(filePath)
        ..createSync(recursive: true)
        ..writeAsStringSync(info.codeBuild(package: package));
    });
    return null;
  }
}

class LibInfo {
  final String name;
  final String version;

  LibInfo({required this.name, required this.version});
}

Builder projectBuilder(BuilderOptions options) =>
    LibraryBuilder(ProjectGenerator(), generatedExtension: ".project.dart");
